﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using UnityEngine.AI;

public class DetectiveScript : MonoBehaviour {

    //What are the stats of this detective?
    public Detective DetectiveStats;

    //Reference to a UI object that defines the state the detective is in
    public Text StateText;

    //What state is the detective in? 
    public int State;

    //Get a reference to the EvidenceHandler in the level.
    EvidenceHandler _handler;

    //
    NavMeshAgent _agent;

    //Get a reference to the Notebook.
    Notebook _notebook;

	// Use this for initialization
	void Start ()
    {
        //Init the state.
        State = 0;

        //Set the handler reference to the handler in the game world.
        _handler = FindObjectOfType<EvidenceHandler>() as EvidenceHandler;

        _agent = GetComponent<NavMeshAgent>();

        _notebook = FindObjectOfType<Notebook>() as Notebook;
	}
	
	// Update is called once per frame
	void Update ()
    {
        displayState();
	}

    /// <summary>
    /// Displays the state that the detective is in.
    /// </summary>
    void displayState()
    {
        switch(State)
        {
            case 0:
                StateText.text = "Idle State";
                break;
            case 1:
                break;
            case 2:
                StateText.text = "Interrogating";
                break;
            default:
                State = 0;
                break;
        }
    }

    /// <summary>
    /// Searches any room given with the detective who calls the function.
    /// </summary>
    /// <param name="r">A Reference to the room that we are attempting to search</param>
    public void SearchRoom(RoomScript r)
    {
        //Sets up some local variables to calculate the time that the search is going to take.
        float timeToSearch, roomSize;

        //Calculates the size of the room given by public variables.
        roomSize = r.thisRoom.Dimensions.x * r.thisRoom.Dimensions.y;

        //Does the math to calculate search time.
        timeToSearch = ((110 - DetectiveStats.SearchAbility) / 10.0f) * roomSize;

        MoveToRoom(r);

        //Begins the coroutine that handles timing the search
        StartCoroutine(StartCountdown(timeToSearch, r));
    }

    /// <summary>
    /// Searches any room given with the detective who calls the function.
    /// </summary>
    /// <param name="r">A Reference to the room that we are attempting to search</param>
    public void Interrogate(WitnessScript w)
    {
        //Sets up some local variables to calculate the time that the search is going to take.
        float timeToSearch;

        //Calculates the time for this detective to interrogate this witness.

        //Does the math to calculate search time.
        timeToSearch = 5.0f;

        //Begins the coroutine that handles timing the search
        StartCoroutine(StartCountdown(timeToSearch, w));
    }

    /// <summary>
    /// A Co-Routine that handles timing the search.
    /// </summary>
    /// <param name="t">The time that the search is going to take.</param>
    /// <param name="r">Reference to the room we are searching.</param>
    /// <returns></returns>
    public IEnumerator StartCountdown(float t, RoomScript r)
    {
        float timer = t;

        //Sets state to the searching mode.
        State = 1;

        //While the timer is above zero, display the time remaining on the screen (Truncated to 2DP) and then wait that delay in this co-routine while reducing that frame-time from the timer.
        while (timer > 0)
        {
            StateText.text = "Searching... " + timer.ToString("F2") + " Seconds Remaining";
            yield return new WaitForSeconds(Time.deltaTime);
            timer -= Time.deltaTime;
        }

        Evidence[] evidenceList = r.gameObject.GetComponents<Evidence>();

        for (int i = 0; i < evidenceList.Length; i++)
        {
            if (evidenceList[i].type == Evidence.evidenceType.Clue)
            {
                _handler.foundClues.Add(evidenceList[i]);
                _notebook.AddClues(evidenceList[i]);
            }
        }
        State = 0;
    }

    /// <summary>
    /// A Co-Routine that handles timing the interrogation.
    /// </summary>
    /// <param name="t">The time that the search is going to take.</param>
    /// <param name="w">Reference to the witness we are interrogating.</param>
    /// <returns></returns>
    public IEnumerator StartCountdown(float t, WitnessScript w)
    {
        float timer = t;

        //Sets state to the searching mode.
        State = 1;

        //While the timer is above zero, display the time remaining on the screen (Truncated to 2DP) and then wait that delay in this co-routine while reducing that frame-time from the timer.
        while (timer > 0)
        {
            StateText.text = "Searching... " + timer.ToString("F2") + " Seconds Remaining";
            yield return new WaitForSeconds(Time.deltaTime);
            timer -= Time.deltaTime;
        }

        Object[] evidenceList = w.gameObject.GetComponents<Evidence>();

        for (int i = 0; i < evidenceList.Length; i++)
        {
            Evidence e = evidenceList[i] as Evidence;

            if (e.type == Evidence.evidenceType.Statement)
            {
                _handler.foundStatements.Add(e);
                _notebook.AddStatements(e);
            }
        }

        // Add line to pull clues into UI.


        //Resets the state.
        State = 0;
    }

    public void MoveToRoom(RoomScript room)
    {
        _agent.SetDestination(room.searchLocation.position);
    }

    public void MoveToLocation(Vector3 endPosition)
    {
        _agent.SetDestination(endPosition);
    }
}
